package haveubreak.ihm;

import haveubreak.Configuration;
import haveubreak.GroupMember;
import haveubreak.Handler;
import haveubreak.HaveubreakEvent;
import haveubreak.HaveubreakListener;
import haveubreak.Main;
import haveubreak.Statut;
import haveubreak.msg.DemandePause;
import java.awt.Color;
import java.awt.Component;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileWriter;
import java.util.Calendar;
import java.util.Properties;
import javax.swing.DefaultListModel;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import org.jgroups.Message;

public class FrameMain extends JFrame implements HaveubreakListener {

	private static final long serialVersionUID = 5765503711977144821L;

	private Configuration config;
    
    private FrameProposerPause subFrame = null;
    
    private TrayIcon trayIcon = null;

    private JTextPane txtPane = null;
    private StyledDocument console = null;

    private Style styleDefault = null;
    private Style styleWelcome = null;
    private Style styleMessages = null;
    private Style styleEvents = null;
    private Style styleError = null;
    private Style styleArrival = null;
    
    public FrameMain(Configuration config) {
        this.config = config;
        
        initComponents();
        setTitle(config.getApplicationName());
        buttonAction.setIcon(Main.ICON_PAUSE);
        setIconImage(Main.ICON_APPLI.getImage());
        listMembers.setModel(new DefaultListModel());
        fieldNickname.setText(config.getNickName());

        // Console
        txtPane = new JTextPane();
        txtPane.setEditable(false);
        styleDefault = txtPane.getStyle("default");
        styleWelcome = txtPane.addStyle("welcome", styleDefault);
        StyleConstants.setForeground(styleWelcome, Color.GRAY);
        styleMessages = txtPane.addStyle("messages", styleDefault);
        StyleConstants.setForeground(styleMessages, Color.BLACK);
        styleEvents = txtPane.addStyle("events", styleDefault);
        StyleConstants.setForeground(styleEvents, Color.BLUE);
        styleError = txtPane.addStyle("error", styleDefault);
        StyleConstants.setForeground(styleError, Color.RED);
        styleArrival = txtPane.addStyle("error", styleDefault);
        StyleConstants.setForeground(styleArrival, Color.ORANGE);
        StyleConstants.setItalic(styleArrival, true);
        console = (StyledDocument) txtPane.getDocument();
        panelFieldConsole.add(new JScrollPane(txtPane));

        initWindowBehavior();
        
        initListMembers();
        
        configChannelName.setText(config.getChannelName());
        configShowPopupDepart.setSelected(config.isShowTrayPopupDeparts());
        configShowPopupRetours.setSelected(config.isShowTrayPopupRetours());
        configShowPopupDemandes.setSelected(config.isShowTrayPopupDemandes());
        configMinimizeInTray.setSelected(config.isMinimizeInTray());
        configApplicationName.setText(config.getApplicationName());
        
        initSystemTrayIcon();
        
        writeTheDay();

        fieldMsg.addKeyListener(new KeyAdapter() {
            @Override
            public void keyTyped(KeyEvent e) {
                if (e.getKeyChar() == 10) {
                    sendTypedMessage();
                }
            }
        });

        fieldNickname.addKeyListener(new KeyAdapter() {
            @Override
            public void keyTyped(KeyEvent e) {
                if (e.getKeyChar() == 10) {
                    fieldMsg.requestFocus();
                }
            }
        });

        fieldNickname.addFocusListener(new FocusAdapter() {
            @Override
            public void focusLost(FocusEvent e) {
                changeNickname();
            }
        });
    }

    public void demandeDePause(String msg, boolean mandatory, String when) {
        Handler.getInstance().send(new DemandePause(msg, mandatory, when));
        Handler.getInstance().setMyCurrentStatut(Statut.AttenteDePause);
    }

    public void changeNickname() {
        String nick = fieldNickname.getText().trim();
        if (nick.equals(config.getNickName())) {
            fieldNickname.setText(config.getNickName());
            return;
        }
        if (!nick.isEmpty()) {
            if (nick.length() > 2) {
                config.setNickName(nick);
                Handler.getInstance().nicknameChanged();
                return;
            }
        }
        fieldNickname.setText(config.getNickName());
    }

    public boolean initConnection() {
        
        buttonAction.setEnabled(false);
        buttonProposerPause.setEnabled(false);
        
        Exception ex = Handler.getInstance().open();
        
        if (ex != null) {
            ex.printStackTrace();
            JOptionPane.showMessageDialog(null, ex.getClass().getSimpleName()
                    +" : "+ex.getMessage(),
                "Erreur de connexion", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        
        return true;
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel2 = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        listMembers = new javax.swing.JList();
        fieldMsg = new javax.swing.JTextField();
        buttonSendMsg = new javax.swing.JButton();
        panelFieldConsole = new javax.swing.JPanel();
        buttonRaz = new javax.swing.JButton();
        buttonSaveLog = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        configChannelName = new javax.swing.JTextField();
        configMinimizeInTray = new javax.swing.JCheckBox();
        buttonAppliquerConfig = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        configShowPopupDemandes = new javax.swing.JCheckBox();
        configShowPopupDepart = new javax.swing.JCheckBox();
        configShowPopupRetours = new javax.swing.JCheckBox();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        configApplicationName = new javax.swing.JTextField();
        buttonProposerPause = new javax.swing.JButton();
        buttonAction = new javax.swing.JButton();
        fieldNickname = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Have a break!");
        setResizable(false);
        getContentPane().setLayout(null);

        jPanel2.setBackground(new java.awt.Color(255, 255, 255));

        jPanel3.setBackground(new java.awt.Color(255, 255, 255));

        jScrollPane1.setViewportView(listMembers);

        buttonSendMsg.setIcon(Main.ICON_MESSAGE);
        buttonSendMsg.setToolTipText("Envoyer un message");
        buttonSendMsg.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonSendMsgActionPerformed(evt);
            }
        });

        panelFieldConsole.setBackground(new java.awt.Color(255, 255, 255));
        panelFieldConsole.setLayout(new java.awt.BorderLayout());

        buttonRaz.setIcon(Main.ICON_RAZ);
        buttonRaz.setToolTipText("Nettoyer la console");
        buttonRaz.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonRazActionPerformed(evt);
            }
        });

        buttonSaveLog.setIcon(Main.ICON_SAVE);
        buttonSaveLog.setToolTipText("Enregistrer le contenu de la console");
        buttonSaveLog.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonSaveLogActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 143, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel3Layout.createSequentialGroup()
                        .addComponent(fieldMsg, javax.swing.GroupLayout.PREFERRED_SIZE, 254, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(buttonSendMsg, javax.swing.GroupLayout.PREFERRED_SIZE, 31, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 8, Short.MAX_VALUE)
                        .addComponent(buttonRaz, javax.swing.GroupLayout.PREFERRED_SIZE, 35, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(buttonSaveLog, javax.swing.GroupLayout.PREFERRED_SIZE, 32, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(panelFieldConsole, javax.swing.GroupLayout.DEFAULT_SIZE, 376, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel3Layout.createSequentialGroup()
                        .addComponent(panelFieldConsole, javax.swing.GroupLayout.PREFERRED_SIZE, 287, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(buttonRaz, javax.swing.GroupLayout.DEFAULT_SIZE, 28, Short.MAX_VALUE)
                            .addComponent(buttonSaveLog, javax.swing.GroupLayout.DEFAULT_SIZE, 28, Short.MAX_VALUE)
                            .addComponent(buttonSendMsg, javax.swing.GroupLayout.DEFAULT_SIZE, 28, Short.MAX_VALUE)
                            .addComponent(fieldMsg, javax.swing.GroupLayout.DEFAULT_SIZE, 28, Short.MAX_VALUE)))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 321, Short.MAX_VALUE))
                .addContainerGap())
        );

        jTabbedPane1.addTab("Groupe", jPanel3);

        jPanel4.setBackground(new java.awt.Color(255, 255, 255));

        jLabel1.setText("Groupe à connecter :");

        configMinimizeInTray.setBackground(new java.awt.Color(255, 255, 255));
        configMinimizeInTray.setText("Minimiser l'application dans la barre des tâches");

        buttonAppliquerConfig.setIcon(Main.ICON_OK);
        buttonAppliquerConfig.setText("Appliquer");
        buttonAppliquerConfig.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonAppliquerConfigActionPerformed(evt);
            }
        });

        jPanel1.setBackground(new java.awt.Color(255, 255, 255));
        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Infobulles"));

        configShowPopupDemandes.setBackground(new java.awt.Color(255, 255, 255));
        configShowPopupDemandes.setText("Afficher les demandes de pause");

        configShowPopupDepart.setBackground(new java.awt.Color(255, 255, 255));
        configShowPopupDepart.setText("Afficher les départs en pause");

        configShowPopupRetours.setBackground(new java.awt.Color(255, 255, 255));
        configShowPopupRetours.setText("Afficher les retours de pause");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(configShowPopupDepart, javax.swing.GroupLayout.DEFAULT_SIZE, 481, Short.MAX_VALUE)
                    .addComponent(configShowPopupDemandes, javax.swing.GroupLayout.DEFAULT_SIZE, 481, Short.MAX_VALUE)
                    .addComponent(configShowPopupRetours, javax.swing.GroupLayout.DEFAULT_SIZE, 481, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(configShowPopupDemandes)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(configShowPopupDepart)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(configShowPopupRetours)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jLabel2.setFont(new java.awt.Font("Tahoma", 2, 11)); // NOI18N
        jLabel2.setText("Version 1.4 beta       Juillet 09");

        jLabel3.setForeground(new java.awt.Color(102, 102, 102));
        jLabel3.setIcon(Main.ICON_WARN);
        jLabel3.setText("Pour changer de groupe ou bien enregistrer votre surnom, cliquer sur Appliquer");

        jLabel4.setText("Nom de l'application :");

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(configMinimizeInTray, javax.swing.GroupLayout.DEFAULT_SIZE, 529, Short.MAX_VALUE)
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel4Layout.createSequentialGroup()
                        .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 299, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 82, Short.MAX_VALUE)
                        .addComponent(buttonAppliquerConfig, javax.swing.GroupLayout.PREFERRED_SIZE, 126, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(28, 28, 28))
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap(30, Short.MAX_VALUE))
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addContainerGap(154, Short.MAX_VALUE))
                    .addGroup(jPanel4Layout.createSequentialGroup()
                        .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(jLabel4, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 146, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(configApplicationName)
                            .addComponent(configChannelName, javax.swing.GroupLayout.DEFAULT_SIZE, 311, Short.MAX_VALUE))
                        .addGap(66, 66, 66))))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 24, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(configChannelName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(configApplicationName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 47, Short.MAX_VALUE)
                .addComponent(configMinimizeInTray)
                .addGap(18, 18, 18)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel3)
                .addGap(22, 22, 22)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(buttonAppliquerConfig, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        jTabbedPane1.addTab("Configuration", jPanel4);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 550, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 368, Short.MAX_VALUE)
                .addContainerGap())
        );

        getContentPane().add(jPanel2);
        jPanel2.setBounds(0, 60, 570, 390);

        buttonProposerPause.setText("Proposer une pause");
        buttonProposerPause.setEnabled(false);
        buttonProposerPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonProposerPauseActionPerformed(evt);
            }
        });
        getContentPane().add(buttonProposerPause);
        buttonProposerPause.setBounds(100, 10, 160, 40);

        buttonAction.setBorder(null);
        buttonAction.setEnabled(false);
        buttonAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonActionActionPerformed(evt);
            }
        });
        getContentPane().add(buttonAction);
        buttonAction.setBounds(10, 10, 70, 40);

        fieldNickname.setBackground(UIManager.getColor("Panel.background"));
        fieldNickname.setFont(new java.awt.Font("Tahoma", 0, 14));
        fieldNickname.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        fieldNickname.setText("jTextField1");
        fieldNickname.setBorder(null);
        getContentPane().add(fieldNickname);
        fieldNickname.setBounds(290, 10, 270, 40);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-577)/2, (screenSize.height-479)/2, 577, 479);
    }// </editor-fold>//GEN-END:initComponents

private void buttonSendMsgActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonSendMsgActionPerformed
    sendTypedMessage();
}//GEN-LAST:event_buttonSendMsgActionPerformed

private void buttonProposerPauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonProposerPauseActionPerformed

    if (subFrame == null) subFrame = new FrameProposerPause(this);
    subFrame.reinit();
    subFrame.setVisible(true);
    
}//GEN-LAST:event_buttonProposerPauseActionPerformed

private void buttonActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonActionActionPerformed

    Handler handler = Handler.getInstance();
    
    if (handler.getMyCurrentStatut().equals(Statut.Travail)) {
        buttonAction.setIcon(Main.ICON_PLAY);
        buttonProposerPause.setEnabled(false);
        handler.setMyCurrentStatut(Statut.Pause);
    }
    
    else if (handler.getMyCurrentStatut().equals(Statut.AttenteDePause)) {
        buttonAction.setIcon(Main.ICON_PLAY);
        buttonProposerPause.setEnabled(false);
        handler.setMyCurrentStatut(Statut.Pause);
    }
    else {
        buttonAction.setIcon(Main.ICON_PAUSE);
        buttonProposerPause.setEnabled(true);
        handler.setMyCurrentStatut(Statut.Travail);
    }
    
}//GEN-LAST:event_buttonActionActionPerformed

private void buttonAppliquerConfigActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonAppliquerConfigActionPerformed
    
    if (!configChannelName.getText().equals(config.getChannelName())) {
        if (configChannelName.getText().trim().isEmpty()) {
            JOptionPane.showMessageDialog(null, "Le nom de channel ne doit pas être vide!",
                    "Erreur de configuration", JOptionPane.ERROR_MESSAGE);
            return;
        }
        else {
            config.setChannelName(configChannelName.getText().trim());
            initConnection();
        }
    }
    
    config.setMinimizeInTray(configMinimizeInTray.isSelected());
    config.setShowTrayPopupDemandes(configShowPopupDemandes.isSelected());
    config.setShowTrayPopupDeparts(configShowPopupDepart.isSelected());

    config.setApplicationName(configApplicationName.getText().trim());

    setTitle(config.getApplicationName());

    saveConfig();
    
}//GEN-LAST:event_buttonAppliquerConfigActionPerformed

private void buttonRazActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonRazActionPerformed
    txtPane.setText("");
}//GEN-LAST:event_buttonRazActionPerformed

private void buttonSaveLogActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonSaveLogActionPerformed

    File fichier = null;
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setSelectedFile(new File("./sauvegarde log.txt"));

    if (fileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
        fichier = fileChooser.getSelectedFile();
    }
    if (fichier != null) {
        if (fichier.exists()) {
            if (JOptionPane.showConfirmDialog(null,
                    "Le fichier existe déja. Remplacer le fichier ?")
                    == JOptionPane.YES_OPTION) {
                fichier.delete();
            } else {
                return;
            }
        }
        try {
            saveLogs(fichier);
            JOptionPane.showMessageDialog(null, "Les logs ont été enregistrés sous :\n"
                    +fichier.getAbsolutePath(), "Enregistrement terminé",
                    JOptionPane.INFORMATION_MESSAGE);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, "Impossible d'enregistrer les logs.\n" +
                    ex.getClass().getSimpleName()+" : "+ex.getMessage(),
                    "Erreur d'enregistrement", JOptionPane.ERROR_MESSAGE);
            ex.printStackTrace();
        }
    }

}//GEN-LAST:event_buttonSaveLogActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton buttonAction;
    private javax.swing.JButton buttonAppliquerConfig;
    private javax.swing.JButton buttonProposerPause;
    private javax.swing.JButton buttonRaz;
    private javax.swing.JButton buttonSaveLog;
    private javax.swing.JButton buttonSendMsg;
    private javax.swing.JTextField configApplicationName;
    private javax.swing.JTextField configChannelName;
    private javax.swing.JCheckBox configMinimizeInTray;
    private javax.swing.JCheckBox configShowPopupDemandes;
    private javax.swing.JCheckBox configShowPopupDepart;
    private javax.swing.JCheckBox configShowPopupRetours;
    private javax.swing.JTextField fieldMsg;
    private javax.swing.JTextField fieldNickname;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JList listMembers;
    private javax.swing.JPanel panelFieldConsole;
    // End of variables declaration//GEN-END:variables

    public void sendTypedMessage() {
        if (!Handler.getInstance().isConnected()) {
            writeConsoleLn("La connexion n'est pas établie, impossible d'envoyer un message.", styleError);
            return;
        }
        String txt = fieldMsg.getText();
        if (!txt.trim().isEmpty()) {
            Handler.getInstance().send(txt);
        }
        fieldMsg.setText("");
        fieldMsg.requestFocus();
    }
    
    public void receivedEvent(final HaveubreakEvent event, final Object data) {
        System.out.println("Event "+event+" : "+data+" "+
                (data instanceof GroupMember ? ((GroupMember) data).getAddr().toString() : ""));
        
        switch (event) {
            
            case CONNECTED :
                buttonProposerPause.setEnabled(true);
                buttonAction.setEnabled(true);
                break;
                
            case DISCONNECTED :
                buttonProposerPause.setEnabled(false);
                buttonAction.setEnabled(false);
                listMembers.setModel(new DefaultListModel());
                listMembers.repaint();
                break;
                
            case MEMBER_JOIN :
                final DefaultListModel model1 = (DefaultListModel) listMembers.getModel();
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        model1.addElement((GroupMember) data);
                        listMembers.repaint();
                    }
                });
                break;
                
            case MEMBER_LOST :
                final DefaultListModel model2 = (DefaultListModel) listMembers.getModel();
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        model2.removeElement((GroupMember) data);
                        listMembers.repaint();
                    }
                });
                GroupMember member0b = (GroupMember) data;
                StringBuilder sb0b = new StringBuilder();
                sb0b.append("** ");
                sb0b.append(member0b.toString());
                sb0b.append(" vient de partir.");
                writeConsoleTime(sb0b.toString(), styleArrival);
                break;
                
            case MESSAGE_RECEIVED :
                Message msg = (Message) data;
                GroupMember member1 = Handler.getInstance().getMemberByAddr(msg.getSrc());
                StringBuilder sb1 = new StringBuilder();
                sb1.append(member1.toString());
                sb1.append("> ");
                sb1.append((String) msg.getObject());
                writeConsoleTime(sb1.toString(), styleMessages);
                break;
                
            case MEMBER_STATUT_CHANGED :
                GroupMember member2 = (GroupMember) data;
                if (member2.getStatut().equals(Statut.Travail)) {
                    writeConsoleTime("** "+member2+" est à son travail.", styleEvents);
                    /// Show tray popup message
                    if (trayIcon != null && config.isShowTrayPopupRetours()
                            && !Handler.getInstance().isMe(member2)) {
                        trayIcon.displayMessage(
                           config.getApplicationName(),
                           member2+" est au travail.",
                           TrayIcon.MessageType.INFO);
                    }
                }
                else if (member2.getStatut().equals(Statut.Pause)) {
                    writeConsoleTime("** "+member2+" est parti en pause.", styleEvents);
                    /// Show tray popup message
                    if (trayIcon != null && config.isShowTrayPopupDeparts()
                            && !Handler.getInstance().isMe(member2)) {
                        trayIcon.displayMessage(
                           config.getApplicationName(),
                           member2+" vient de partir en pause",
                           TrayIcon.MessageType.INFO);
                    }
                }
                listMembers.repaint();
                break;

            case MEMBER_NICKNAME_CHANGED :
                GroupMember member4 = (GroupMember) data;
                writeConsoleTime("** Changement de nickname : "+member4.getNickName()
                        +" ("+member4.getUserName()+")", styleEvents);
                listMembers.repaint();
                break;
                
            case MEMBER_DEMANDE_PAUSE :
                Object[] datas = (Object[]) data;
                DemandePause demande = (DemandePause) datas[1];
                GroupMember member3 = (GroupMember) datas[0];
                // Write console line
                StringBuilder sb2 = new StringBuilder();
                sb2.append("** ");
                sb2.append(member3.toString());
                sb2.append(" demande une pause ");
                if (demande.isMandatory()) {
                    sb2.append("OBLIGATOIRE ");
                }
                sb2.append(demande.getWhen().equals("ASAP") ? "ASAP" : "dans "+demande.getWhen());
                if (!demande.getMsg().isEmpty()) {
                    sb2.append(" : ");
                    sb2.append(demande.getMsg());
                }
                writeConsoleTime(sb2.toString(), styleEvents);
                /// Show tray popup message
                if (trayIcon != null && config.isShowTrayPopupDemandes() &&
                        !Handler.getInstance().isMe(member3)) {
                    StringBuilder sb3 = new StringBuilder();
                    sb3.append(member3.toString());
                    sb3.append(" veut une pause ");
                    sb3.append(demande.getWhen().equals("ASAP") ? "ASAP" : "dans "+demande.getWhen());
                    if (!demande.getMsg().isEmpty()) {
                        sb3.append(" : ");
                        sb3.append(demande.getMsg());
                    }
                    trayIcon.displayMessage(
                       config.getApplicationName(),
                       sb3.toString(),
                       TrayIcon.MessageType.INFO);
                }
                break;
        }

    }

    private void saveLogs(File file) throws Exception {
        FileWriter fw = new FileWriter(file);
        fw.write(txtPane.getText());
        fw.close();
    }
    
    private void writeConsoleTime(String msg, Style style) {
        StringBuilder sb = new StringBuilder();
        sb.append('\n');
        sb.append('[');
        Calendar c = Calendar.getInstance();
        if (c.get(Calendar.HOUR_OF_DAY) < 10) {
            sb.append('0');
        }
        sb.append(c.get(Calendar.HOUR_OF_DAY));
        sb.append(':');
        if (c.get(Calendar.MINUTE) < 10) {
            sb.append('0');
        }
        sb.append(c.get(Calendar.MINUTE));
        sb.append(']');
        sb.append(' ');
        sb.append(msg);
        writeConsoleLn(sb.toString(), style);
    }

    private boolean writeConsoleLn(String msg, Style style) {
        try {
            console.insertString(console.getLength(), msg, style);
            txtPane.setCaretPosition(console.getLength());
            return true;
        } catch (Exception ex) {
            return false;
        }
    }

    private void initListMembers() {
        listMembers.setModel(new DefaultListModel());
        listMembers.setCellRenderer(new ListCellRenderer() {

            public Component getListCellRendererComponent(
                    JList list,
                    Object value,
                    int index,
                    boolean isSelected,
                    boolean cellHasFocus) {
                
                GroupMember member = (GroupMember) value;
                
                JLabel label = new JLabel(member.toString());
                
                if (!member.isNamed()) {
                    label.setIcon(Main.ICON_WARN);
                }
                else {
                    switch (member.getStatut()) {
                        case Pause :
                            label.setIcon(Main.ICON_OK);
                            break;
                        case AttenteDePause :
                            label.setIcon(Main.ICON_HELP);
                            break;
                        case Travail :
                            label.setIcon(Main.ICON_CANCEL);
                            break;
                    }
                }
                
                label.setForeground(member.isNamed() ? Color.BLACK : Color.GRAY);
                label.setBackground(isSelected ? Color.LIGHT_GRAY : Color.WHITE);
                
                label.setToolTipText(member.getNickName()+" ("+member.getUserName()
                        +"@"+member.getAddr()+")");
                
                return label;
                
            }
            
        });
    }

    private void initSystemTrayIcon() {
        if (SystemTray.isSupported()) {
        	
//        	PopupMenu menu = new PopupMenu();
        	/*MenuItem quit = new MenuItem("Quitter");
        	quit.addActionListener(new ActionListener() {
        		@Override
        		public void actionPerformed(ActionEvent e) {
        			if (trayIcon != null) {
        				SystemTray.getSystemTray().remove(trayIcon);
        			}
        			dispose();
        			System.exit(0);
        		}
        	});
        	menu.add(quit);*/
        	
        	/*final MenuItem action = new MenuItem("Partir en pause");
        	action.addActionListener(new ActionListener() {
        		@Override
        		public void actionPerformed(ActionEvent e) {
        			
        		}
        	});
        	menu.add(action);*/
        	
            trayIcon = new TrayIcon(Main.ICON_APPLI.getImage(),
            		config.getApplicationName(), null);
            try {
                SystemTray.getSystemTray().add(trayIcon);
                trayIcon.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        setVisible(true);
                    }
                });
            } catch (Exception ex) {
                trayIcon = null;
            }
        }
    }

    private void initWindowBehavior() {
        //setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {

            @Override
            public void windowIconified(WindowEvent e) {
                if (trayIcon != null && config.isMinimizeInTray()) {
                    setExtendedState(JFrame.NORMAL);
                    setVisible(false);
                }
            }

            @Override
            public void windowClosed(WindowEvent e) {
                if (trayIcon != null) {
                    SystemTray.getSystemTray().remove(trayIcon);
                }
            }
            
        });
    }

    private void writeTheDay() {
        Calendar c = Calendar.getInstance();
        StringBuilder sb = new StringBuilder();
        sb.append("Nous sommes le ");
        switch (c.get(Calendar.DAY_OF_WEEK)) {
            case Calendar.MONDAY : sb.append("lundi"); break;
            case Calendar.TUESDAY : sb.append("mardi"); break;
            case Calendar.WEDNESDAY : sb.append("mercredi"); break;
            case Calendar.THURSDAY : sb.append("jeudi"); break;
            case Calendar.FRIDAY : sb.append("vendredi"); break;
            case Calendar.SATURDAY : sb.append("samedi"); break;
            case Calendar.SUNDAY : sb.append("dimanche"); break;
        }
        sb.append(' ');
        sb.append(c.get(Calendar.DAY_OF_MONTH));
        sb.append(' ');
        switch (c.get(Calendar.MONTH)) {
            case Calendar.JANUARY : sb.append("janvier"); break;
            case Calendar.FEBRUARY : sb.append("février"); break;
            case Calendar.MARCH : sb.append("mars"); break;
            case Calendar.APRIL : sb.append("avril"); break;
            case Calendar.MAY : sb.append("mai"); break;
            case Calendar.JUNE : sb.append("juin"); break;
            case Calendar.JULY : sb.append("juillet"); break;
            case Calendar.AUGUST : sb.append("août"); break;
            case Calendar.SEPTEMBER : sb.append("septembre"); break;
            case Calendar.OCTOBER : sb.append("octobre"); break;
            case Calendar.NOVEMBER : sb.append("novembre"); break;
            case Calendar.DECEMBER : sb.append("décembre"); break;
        }
        sb.append(' ');
        sb.append(c.get(Calendar.YEAR));
        writeConsoleLn(sb.toString(), styleWelcome);
    }

    private boolean saveConfig() {
        Properties prop = new Properties();
        prop.put("user.nickname", config.getNickName());
        prop.put("channel.name", config.getChannelName());
        prop.put("minimizeintray", ""+config.isMinimizeInTray());
        prop.put("popup.demandes", ""+config.isShowTrayPopupDemandes());
        prop.put("popup.departs", ""+config.isShowTrayPopupDeparts());
        prop.put("popup.retours", ""+config.isShowTrayPopupRetours());
        prop.put("appli.name", ""+config.getApplicationName());
        try {
            prop.store(new FileWriter(new File("./haveubreak.properties")), "HaveuBreak config file");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return false;
    }
    
}
